@@ -3,6 +3,7 @@ Variogram class
 """
 import copy
 import warnings
+from typing import Iterable, Callable, Union, Tuple
 
 import numpy as np
 from pandas import DataFrame
@@ -101,7 +102,7 @@ class Variogram(object):
             scipy.spatial.distance.pdist. Additional parameters are not (yet)
             passed through to pdist. These are accepted by pdist for some of
             the metrics. In these cases the default values are used.
-        bin_func : str
+        bin_func : str | Callable | Iterable
             .. versionchanged:: 0.3.8
                 added 'fd', 'sturges', 'scott', 'sqrt', 'doane'
             .. versionchanged:: 0.3.9
@@ -131,13 +132,14 @@ class Variogram(object):
         normalize : bool
             Defaults to False. If True, the independent and dependent
             variable will be normalized to the range [0,1].
-        fit_method : str
+        fit_method : str | None
             .. versionchanged:: 0.3.10
                 Added 'ml' and 'custom'
 
             String identifying the method to be used for fitting the
-            theoretical variogram function to the experimental. More info is
-            given in the Variogram.fit docs. Can be one of:
+            theoretical variogram function to the experimental. If None
+            is passed, the fit does not run. More info is given in the
+            Variogram.fit docs. Can be one of:
 
                 * 'lm': Levenberg-Marquardt algorithm for unconstrained
                   problems. This is the faster algorithm, yet is the fitting of
@@ -311,10 +313,13 @@ class Variogram(object):
         self._bin_func = None
         self._groups = None
         self._bins = None
+        self._bin_count = None
         self.set_bin_func(bin_func=bin_func)
 
-        # Needed for harmonized models
-        self.preprocessing(force=True)
+        # Needed for harmonized models, no need to
+        # run if there is no fit
+        if fit_method is not None:
+            self.preprocessing(force=True)
 
         # model can be a function or a string
         self._model = None
@@ -523,12 +528,15 @@ class Variogram(object):
     def bin_func(self, bin_func):
         self.set_bin_func(bin_func=bin_func)
 
-    def set_bin_func(self, bin_func: str):
+    def set_bin_func(self, bin_func: Union[str, Iterable, Callable[[np.ndarray, float, float], Tuple[np.ndarray, float]]]):
         r"""Set binning function
 
         Sets a new binning function to be used. The new binning method is set
-        by a string identifying the new function to be used. Can be one of:
-        ['even', 'uniform', 'fd', 'sturges', 'scott', 'sqrt', 'doane'].
+        by either a string identifying the new function to be used, or an
+        iterable containing the bin edges, or any function that can compute
+        bins from the distances, number of lags and maximum lag.
+        The string can be one of: ['even', 'uniform', 'fd',
+         'sturges', 'scott', 'sqrt', 'doane'].
         If the number of lag classes should be estimated automatically, it is
         recommended to use ' sturges' for small, normal distributed locations
         and 'fd' or 'scott' for large datasets, where 'fd' is more robust to
@@ -549,9 +557,13 @@ class Variogram(object):
             pass kwargs to the binning functions is now implemented as
             a instance method, to make it pickleable.
 
+        .. versionchanged:: 0.6.5
+            added iterable and function as arguments to allow for custom
+            bins.
+
         Parameters
         ----------
-        bin_func : str
+        bin_func : str | Iterable | Callable
             Can be one of:
 
                 * 'even'
@@ -671,20 +683,34 @@ class Variogram(object):
 
                 # use the wrapper for all but even and uniform
                 self._bin_func = self._bin_func_wrapper
+            bin_func_name = bin_func
 
-        elif callable(bin_func):  # pragma: no cover
+        elif isinstance(bin_func, Callable):  # pragma: no cover
             self._bin_func = bin_func
-            bin_func = 'custom'
+            bin_func_name = 'custom_func'
+
+        elif isinstance(bin_func, Iterable):
+            self._bin_func = None
+            bin_func_name = 'custom_bin_edges'
 
         else:
-            raise AttributeError('bin_func has to be of type string.')
+            raise AttributeError('bin_func has to be of type string, iterable or callable.')
 
         # store the name
-        self._bin_func_name = bin_func
+        self._bin_func_name = bin_func_name
 
-        # reset groups and bins
+        # reset groups, bins and bin count
         self._groups = None
-        self._bins = None
+        self._bin_count = None
+
+        if isinstance(bin_func, str) or isinstance(bin_func, Callable):
+            self._bins = None
+        # if the input is an iterable with bin edges, we need to write self._bins here
+        else:
+            self._bins = bin_func
+            self._maxlag = max(bin_func)
+            self._n_lags = sum(1 for e in bin_func)
+
         self.cof, self.cov = None, None
 
     def _bin_func_wrapper(self, distances, n, maxlag):
@@ -759,6 +785,7 @@ class Variogram(object):
         self._bins = np.asarray(bins)
 
         # clean the groups as they are not valid anymore
+        self._bin_count = None
         self._groups = None
         self.cov = None
         self.cof = None
@@ -801,13 +828,21 @@ class Variogram(object):
         # if there are no errors, store the passed value
         self._n_lags_passed_value = n
 
-        # reset the groups
+        # reset the groups and bin count
         self._groups = None
+        self._bin_count = None
 
         # reset the fitting
         self.cof = None
         self.cov = None
 
+    @property
+    def bin_count(self):
+
+        if self._bin_count is None:
+            self._bin_count = np.fromiter((g.size for g in self.lag_classes()), dtype=int)
+        return self._bin_count
+
     @property
     def estimator(self):
         return self._estimator
@@ -927,15 +962,31 @@ class Variogram(object):
 
     @property
     def use_nugget(self):
+        """
+        Use a nugget effect on this Variogram instance.
+        If disabled, the automatic fitting procedures will omit the nugget and
+        not use it as a model parameter.
+
+        Note
+        ----
+        If :func:`fit_method <skgstat.Variogram.fit_method>` is set to
+        ``'manual'`` and a nugget parameter is pass to
+        :func:`fit <skgstat.Variogram.fit>`, use_nugget will be set to True.
+
+        Returns
+        -------
+        use_nugget : bool
+
+        """
         return self._use_nugget
 
     @use_nugget.setter
-    def use_nugget(self, nugget):
-        if not isinstance(nugget, bool):
+    def use_nugget(self, enable_nugget):
+        if not isinstance(enable_nugget, bool):
             raise ValueError('use_nugget has to be of type bool.')
 
         # set new value
-        self._use_nugget = nugget
+        self._use_nugget = enable_nugget
 
     @property
     def dist_function(self):
@@ -1044,9 +1095,10 @@ class Variogram(object):
         # reset fitting
         self.cof, self.cov = None, None
 
-        # remove bins
+        # remove bins, groups, and bin count
         self._bins = None
         self._groups = None
+        self._bin_count = None
 
         # set new maxlag
         if value is None:
@@ -1074,6 +1126,10 @@ class Variogram(object):
         * ``'ml'``    - Maximum Likelihood estimation
         * `'manual'`` - Manual fitting by setting the parameters
 
+        .. versionchanged:: 0.6.6
+            Passing `None` will prevent the fitting procedure from
+            running.
+
         See Also
         --------
         scipy.optimize.minimize
@@ -1340,6 +1396,10 @@ class Variogram(object):
         .. versionchanged:: 0.3.10
             added 'ml' and 'custom' method.
 
+        .. versionchanged:: 1.0.1 
+            use_nugget is now flagged implicitly, whenever a nugget > 0 is
+            passed in manual fitting.    
+        
         Parameters
         ----------
         force : bool
@@ -1396,7 +1456,7 @@ class Variogram(object):
         self.preprocessing(force=force)
 
         # load the data
-        x = self.bins
+        x = np.array(self.bins)
         y = self.experimental
 
         # overwrite fit setting if new params are given
@@ -1409,6 +1469,10 @@ class Variogram(object):
         _x = x[~np.isnan(y)]
         _y = y[~np.isnan(y)]
 
+        # check if method is manual and a nugget was passed
+        if self.fit_method == 'manual' and kwargs.get('nugget', False):
+            self.use_nugget = True
+
         # handle harmonized models
         if self._harmonize:
             _x = np.linspace(0, np.max(_x), 100)
@@ -2127,6 +2191,20 @@ class Variogram(object):
 
         return 1 - (term1 / term2)
 
+    @property
+    def aic(self) -> float:
+        """
+        """
+        from skgstat.util.cross_validation import aic
+        return aic(self)
+
+    @property
+    def bic(self) -> float:
+        """
+        """
+        from skgstat.util.cross_validation import bic
+        return bic(self)
+
     def model_deviations(self):
         """Model Deviations
 
